﻿/*
 * Sample Folder Management Assembly
 * Exago, Inc
 * support@exagoinc.com
 * 
 * This file is provided as an example assembly for report folder and report template management within eWebReports.
 * Although this assembly should provide a good starting point, it is recommended that you add/modify code to fully integrate
 * with your system to account for application standards including error handling and database design.  Specifically, the parameters
 * userId and companyId passed to all methods should be used to enforce management permissions on a user and/or company basis.
 * 
 * Please make sure to review/alter this code to make sure it meets your needs before integrating into a production system!
 *
 * ---------------------------------------------------------------------------------------------------------------
 *   PLEASE NOTE
 *   This software is provided to our clients as a convenience "as-is," with no implied warranty.
 * ---------------------------------------------------------------------------------------------------------------
 * 
 * Quick setup:
 * 
 * 1. Create the database table to store the report, folder, and template data (schema info below)
 * 2. Fill in the example database connection info in this file with your own (see member variables eg. "this.server") and build the solution
 * 3. Tell eWebReports to use the assembly for folder management
 *    a) Open the admin console
 *    b) Navigate to General -> Main Settings
 *    c) Change the "Report Path" field to point to the .dll file associated with this project, using the "assembly=X;class=Y" scheme.
 *       eg. assembly=C:\ExagoFolderMgmtAssembly\bin\ExagoFolderMgmtAssembly.dll;class=ExagoFolderMgmtAssembly.ExagoFolderMgmtAssembly
 * 4. Once the Report Path field validates without error, eWebReports will use the database for storage of reports, folders, and templates
 * 
 * 
 * Note: This example was developed and tested against MSSQL Server, however, the queries used in the example 
 * methods provided should work across major database platforms with equivalent schemas.
 * 
 * Database Schema:
 * 
 * ID            int identity...Unique ID
 * companyId     varchar........Company ID of the owner of the report (null for templates)
 * userId        varchar........User ID of the owner of the report (null for templates)
 * name          varchar........Name of the report, template, or folder
 * content       varchar........XML content of the report (null for folders and templates)
 * leaf_flag     bit............True for reports and templates, false for folders
 * readonly_flag bit............True if the report or folder is readonly
 * bin_content   varbinary......Contains the template file data (null if not a template)
 * type          int............0 if report, 1 if template, 2 if theme
 * isDeleted     bit............True if the Report was deleted, false otherwise
 * themeType     varchar........Type of the theme being saveed or loaded (ex. chart, map, express or crosstab)
 * PARENT_ID     int............ID of the parent folder of the report (null for folders, themes, and templates)
 * 
 *
 *  
 * CREATE TABLE Tablename(ID int identity, PARENT_ID int, companyId varchar, userId varchar, name varchar, 
 *                     content varchar, leaf_flag bit, readonly_flag bit, bin_content varbinary(max), type int);
 * 
 */

using System;
using System.Data;
using System.Xml;
using System.IO;
using System.Data.SqlClient;
using System.Collections.Generic;
using WebReports.Api.Common;

namespace ExagoFolderMgmtExamples
{
	public static class TypeIds
	{
		public static int ReportType = 0;
		public static int TemplateType = 1;
		public static int ThemeType = 2;
	}

	public class Entity
	{
		public int Id;
		public string Name;
		public bool LeafFlag;
		public bool ReadOnlyFlag;
		public string Description;
		public List<Entity> Children = new List<Entity>();

		public Entity(int id, string name, bool leafFlag, bool readOnlyFlag, string description)
		{
			this.Id = id;
			this.Name = name;
			this.LeafFlag = leafFlag;
			this.ReadOnlyFlag = readOnlyFlag;
			this.Description = description;
		}
	}

	public class AssemblyExample
	{
		private static char[] tokens = { '\\', '/' };

		// Database connection info (alter to connect to your database)
		private const string server = "SERVER";
		private const string databaseName = "DATABASE";
		private const string dbUserId = "USER";
		private const string password = "PASSOWRD";
		private const string reportTableName = "TABLE";

        #region Cloud Temp Storage Member Variables
        private const string tempStorageTable = "CloudTempStorage";
        #endregion

        private static SqlConnection InitConnection()
		{
			SqlConnection sqlConn = new SqlConnection(String.Format("Server={0};Database={1};uid={2};pwd={3};", AssemblyExample.server, AssemblyExample.databaseName, AssemblyExample.dbUserId, AssemblyExample.password));
			sqlConn.Open();
			return sqlConn;
		}

		// Return XML representing the hierarchy of reports and folders
		public static string GetReportListXml(SessionInfo sessionInfo)
		{


			List<Entity> entities = new List<Entity>();
			Dictionary<int, Entity> entityIdMapping = new Dictionary<int, Entity>();

			// NOTE: In your own implementation, use companyId and userId to enforce visibility permissions on files and folders
            string stmt = String.Format("SELECT name, content, leaf_flag, readonly_flag, ID, PARENT_ID FROM {0} WHERE type='{1}' AND isDeleted = 'false'", AssemblyExample.reportTableName, TypeIds.ReportType);

			List<SqlParameter> sqlParams = new List<SqlParameter>();
			DataSet ds = ExecuteReadCmd(stmt, sqlParams);
			List<DataRow> deleteList = new List<DataRow>();
			bool nodeAdded = false;
			
			// The idea is to loop through the returned dataset adding nodes, until no more can be added.  Since a child may appear
			// in the dataset before its parent and the parent needs to exist in the XmlDoc before adding its child, we can't
			// just loop through the list and add all nodes sequentially.
			while(true)
			{
				nodeAdded = false;

                if (ds == null)
                {
                    return "<entity></entity>";
                }
                
				foreach (DataRow dr in ds.Tables[0].Rows)
				{
					string name = (string)dr["name"];
					int id = (int)dr["ID"];
					string leafFlag = bool.Parse(dr["leaf_flag"].ToString()) == true ? "true" : "false";
					object parentObj = dr["PARENT_ID"];

					if (parentObj.Equals(DBNull.Value))
					{
						// This entity is a root folder
						entities.Add(new Entity(id, name, false, false, null));
						entityIdMapping.Add(id, entities[entities.Count - 1]); // Keep a "pointer" to the node so we can easily access it by ID when we need to add children

						nodeAdded = true;
						deleteList.Add(dr);
					}
					else
					{
						// The entity is a child of a folder.  Find the parent entity and add this entity as a child.
						string parent_id = "" + (int)parentObj;

						if (!entityIdMapping.ContainsKey((int)parentObj))
							continue;

						Entity parent = entityIdMapping[(int)parentObj];

						nodeAdded = true;
						deleteList.Add(dr);

						string description = null;

						if(dr["content"].GetType() != typeof(DBNull))
						{
							XmlDocument reportDoc = new XmlDocument();
							reportDoc.LoadXml((string)dr["content"]);
							XmlNode descriptionNode = reportDoc.SelectSingleNode("/report/main/description");

							if (descriptionNode != null)
								description = descriptionNode.InnerText;
						}

						parent.Children.Add(new Entity(id, name, bool.Parse(leafFlag), false, description));
						entityIdMapping.Add(id, parent.Children[parent.Children.Count - 1]);
					}
				}

				foreach(DataRow dr in deleteList)
					ds.Tables[0].Rows.Remove(dr);

				if (!nodeAdded && ds.Tables[0].Rows.Count > 0)
                {
                    throw new Exception("Table malformatted: Found child with nonexistent parent");
                }
                else if (!nodeAdded)
                {
                    break;
                }

				deleteList.Clear();
			}

			// Recursively sort the entities list (Reports before folders; everything alphabetically)
			AssemblyExample.SortEntities(entities);

			XmlDocument xDoc = new XmlDocument();
			XmlNode rootNode = xDoc.CreateElement("eWebReportsReports");
			xDoc.AppendChild(rootNode);

                        
            string folderxml = AssemblyExample.WriteXml(entities, xDoc, rootNode);

            // Render the entities list as XML to return to the caller
			return folderxml;
		}

		private static void SortEntities(List<Entity> entities)
		{
			foreach (Entity entity in entities)
			{
				entities.Sort((a, b) =>
				{
					if (a.LeafFlag && !b.LeafFlag)
						return -1;
					if (b.LeafFlag && !a.LeafFlag)
						return 1;
					return a.Name.CompareTo(b.Name);
				});

				SortEntities(entity.Children);
			}
		}

		private static string WriteXml(List<Entity> entities, XmlDocument xmlDoc, XmlNode root)
		{
			foreach (Entity entity in entities)
			{
				XmlNode node = xmlDoc.CreateElement("entity");
				root.AppendChild(node);

				XmlAttribute attribute = xmlDoc.CreateAttribute("id");
				attribute.Value = entity.Id.ToString();
				node.Attributes.Append(attribute);
				XmlNode nameNode = xmlDoc.CreateElement("name");
				node.AppendChild(nameNode);
				nameNode.InnerText = entity.Name;
				XmlNode leafNode = xmlDoc.CreateElement("leaf_flag");
				node.AppendChild(leafNode);
				leafNode.InnerText = entity.LeafFlag.ToString();
				XmlNode roNode = xmlDoc.CreateElement("readonly_flag");
				node.AppendChild(roNode);
                roNode.InnerText = entity.ReadOnlyFlag.ToString();

				if (entity.Description != null)
				{
					XmlNode descriptionNode = xmlDoc.CreateElement("description");
					node.AppendChild(descriptionNode);
					descriptionNode.InnerText = entity.Description;
				}

				WriteXml(entity.Children, xmlDoc, node);
			}

			return xmlDoc.InnerXml;
		}

		// Add a report folder (folderName is fully qualified). Folder should not be named to one that already exists
		public static string AddFolder(SessionInfo sessionInfo, string folderName)
		{
            string[] pathParts = folderName.Split(tokens);
			string parentIdStr = GetParentFolderIdString(sessionInfo.CompanyId, sessionInfo.UserId, pathParts, false);

			// NOTE: In your own implementation, use companyId and userId to enforce edit permissions on files and folders
            string stmt = String.Format("INSERT INTO {0} VALUES (  @companyId, @userId, @name, NULL, 'false', 'false', NULL, '{1}', 'false', NULL,{2})", AssemblyExample.reportTableName, TypeIds.ReportType, parentIdStr);

			List<SqlParameter> sqlParams = new List<SqlParameter>();
			sqlParams.Add(new SqlParameter("@companyId", sessionInfo.CompanyId));
			sqlParams.Add(new SqlParameter("@userId", sessionInfo.UserId));
			sqlParams.Add(new SqlParameter("@name", pathParts[pathParts.Length - 1]));

			return ExecuteNQCmd(stmt, sqlParams);
		}

		// Delete a report folder (folderName is fully qualified)
		public static string DeleteFolder(SessionInfo sessionInfo, string folderName)
		{
            string[] pathParts = folderName.Split(tokens);
			string parentIdStr = GetParentFolderIdString(sessionInfo.CompanyId, sessionInfo.UserId, pathParts, true);

			// NOTE: In your own implementation, use companyId and userId to enforce edit permissions on files and folders
            string stmt = String.Format("UPDATE {0} SET isDeleted = 'true' WHERE name=@name AND parent_Id = {1}", AssemblyExample.reportTableName, parentIdStr);

			List<SqlParameter> sqlParams = new List<SqlParameter>();
			sqlParams.Add(new SqlParameter("@name", pathParts[pathParts.Length - 1]));

			return ExecuteNQCmd(stmt, sqlParams);
		}

		// Rename a report folder (folderName is fully qualified). Folder should not be renamed to one that already exists
		public static string RenameFolder(SessionInfo sessionInfo, string oldName, string newName)
		{
            string[] oldPathParts = oldName.Split(tokens);
			string[] newPathParts = newName.Split(tokens);
			string parentIdStr = GetParentFolderIdString(sessionInfo.CompanyId, sessionInfo.UserId, oldPathParts, true);

			// NOTE: In your own implementation, use companyId and userId to enforce edit permissions on files and folders
            string stmt = String.Format("UPDATE {0} SET name = @newName WHERE name=@oldName AND parent_Id ={1}", AssemblyExample.reportTableName, parentIdStr);

			List<SqlParameter> sqlParams = new List<SqlParameter>();
			sqlParams.Add(new SqlParameter("@newName", newPathParts[newPathParts.Length - 1]));
			sqlParams.Add(new SqlParameter("@oldName", oldPathParts[oldPathParts.Length - 1]));

			return ExecuteNQCmd(stmt, sqlParams);
		}

		// Rename a report folder (folderName is fully qualified). Folder should not be renamed to one that already exists
		public static string MoveFolder(SessionInfo sessionInfo, string oldName, string newName)
		{
            string[] oldPathParts = oldName.Split(tokens);
			string[] newPathParts = newName.Split(tokens);
			string stmt = null;
			List<SqlParameter> sqlParams = new List<SqlParameter>();

			// Find the entry we want to rename
			object oldFolderId = null;
			
			for(int i=0; i < oldPathParts.Length; i++)
			{
				if(oldFolderId != null)
                    stmt = String.Format("SELECT ID FROM {0} WHERE name=@name AND PARENT_ID=@parentId", AssemblyExample.reportTableName);
				else
                    stmt = String.Format("SELECT ID FROM {0} WHERE name=@name AND PARENT_ID IS NULL", AssemblyExample.reportTableName);

				sqlParams.Clear();
				sqlParams.Add(new SqlParameter("@name", oldPathParts[i]));

				if (oldFolderId != null)
					sqlParams.Add(new SqlParameter("@parentId", oldFolderId));

				oldFolderId = ExecuteScalarCmd(stmt, sqlParams);
			}

			// Now that we have the ID of the folder we want to move, get the ID of the folder we want to move it to
			object destinationFolderId = null;

			for (int i = 0; i < newPathParts.Length - 1; i++)
			{
				if(destinationFolderId != null)
                    stmt = String.Format("SELECT ID FROM {0} WHERE name=@name AND PARENT_ID=@parentId", AssemblyExample.reportTableName);
				else
                    stmt = String.Format("SELECT ID FROM {0} WHERE name=@name AND PARENT_ID IS NULL", AssemblyExample.reportTableName);

				sqlParams.Clear();
				sqlParams.Add(new SqlParameter("@name", newPathParts[i]));

				if (destinationFolderId != null)
					sqlParams.Add(new SqlParameter("@parentId", destinationFolderId));

				destinationFolderId = ExecuteScalarCmd(stmt, sqlParams);
			}

			
			// Now perform the "move" ie. update teh folder's parent ID to the destination folder's ID
			// NOTE: In your own implementation, use companyId and userId to enforce edit permissions on files and folders
			if(destinationFolderId != null)
                stmt = String.Format("UPDATE {0} SET PARENT_ID = @parentId, name = @newName WHERE ID=@id", AssemblyExample.reportTableName);
			else
                stmt = String.Format("UPDATE {0} SET PARENT_ID = NULL, name = @newName WHERE ID=@id", AssemblyExample.reportTableName);

			sqlParams.Clear();
			sqlParams.Add(new SqlParameter("@id", oldFolderId));

			if (destinationFolderId != null)
				sqlParams.Add(new SqlParameter("@parentId", destinationFolderId));

			sqlParams.Add(new SqlParameter("@newName", newPathParts[newPathParts.Length - 1]));

			return ExecuteNQCmd(stmt, sqlParams);
		}

		// Determines if a folder exists; returns true or false
		public static bool ExistFolder(SessionInfo sessionInfo, string folderName)
		{

			string[] pathParts = folderName.Split(tokens);
			string parentIdStr = GetParentFolderIdString(sessionInfo.CompanyId, sessionInfo.UserId, pathParts, true);

			// NOTE: In your own implementation, use companyId and userId to enforce edit permissions on files and folders
            string stmt = String.Format("SELECT name FROM {0} WHERE isDeleted ='false' AND name=@name AND {1}", AssemblyExample.reportTableName, parentIdStr);

			List<SqlParameter> sqlParams = new List<SqlParameter>();
			sqlParams.Add(new SqlParameter("@name", pathParts[pathParts.Length - 1]));

			object obj = ExecuteScalarCmd(stmt, sqlParams);

			return (obj != null);
		}

		// Gets a report (reportName is fully qualified)
		public static string GetReportXml(SessionInfo sessionInfo, string reportName)
		{
			string[] pathParts = reportName.Split(tokens);
			string parentIdStr = GetParentFolderIdString(sessionInfo.CompanyId, sessionInfo.UserId, pathParts, true);

			// NOTE: In your own implementation, use companyId and userId to enforce read permissions on files and folders
            string stmt = String.Format("SELECT content FROM {0} WHERE name=@name AND {1}", AssemblyExample.reportTableName, parentIdStr);

			List<SqlParameter> sqlParams = new List<SqlParameter>();
			sqlParams.Add(new SqlParameter("@name", pathParts[pathParts.Length - 1]));

			object obj = ExecuteScalarCmd(stmt, sqlParams);

			return obj.ToString();
		}

		// Save a report (reportName is fully qualified)
		public static string SaveReport(SessionInfo sessionInfo, string reportName, string reportXml)
		{
            string stmt = "";
			string[] pathParts = reportName.Split(tokens);
			string parentIdChkStr = GetParentFolderIdString(sessionInfo.CompanyId, sessionInfo.UserId, pathParts, true);
			string parentIdStr = GetParentFolderIdString(sessionInfo.CompanyId, sessionInfo.UserId, pathParts, false);

			// NOTE: In your own implementation, use companyId and userId to enforce edit permissions on files and folders
            string chStmt = String.Format("SELECT name FROM {0} WHERE {1} AND name=@name", AssemblyExample.reportTableName, parentIdChkStr);

			List<SqlParameter> sqlParams = new List<SqlParameter>();
			sqlParams.Add(new SqlParameter("@name", pathParts[pathParts.Length - 1]));

			Object chkObj = ExecuteScalarCmd(chStmt, sqlParams);

			sqlParams.Clear();

			if (chkObj == null)
			{
                stmt = String.Format("INSERT INTO {0} VALUES (@companyId, @userId, @name, @reportXml, 'true', 'false', NULL, '{1}', 'false', NULL, {2})", AssemblyExample.reportTableName, TypeIds.ReportType, parentIdStr);
				sqlParams.Add(new SqlParameter("@companyId", sessionInfo.CompanyId));
				sqlParams.Add(new SqlParameter("@userId", sessionInfo.UserId));
				sqlParams.Add(new SqlParameter("@name", pathParts[pathParts.Length - 1]));
				sqlParams.Add(new SqlParameter("@reportXml", reportXml));
			}
			else
			{
                stmt = String.Format("UPDATE {0} SET content = @reportXml WHERE {1} AND name=@name", AssemblyExample.reportTableName, parentIdChkStr);
				sqlParams.Add(new SqlParameter("@reportXml", reportXml));
				sqlParams.Add(new SqlParameter("@companyId", sessionInfo.CompanyId));
				sqlParams.Add(new SqlParameter("@userId", sessionInfo.UserId));
				sqlParams.Add(new SqlParameter("@name", pathParts[pathParts.Length - 1]));
			}
			return ExecuteNQCmd(stmt, sqlParams);
		}

		// Duplicates a report (reportName is fully qualified). If this method is not provided, SaveReport will be called
		public static string DuplicateReport(SessionInfo sessionInfo, string reportName, string reportXml)
		{
            return SaveReport(sessionInfo, reportName, reportXml);
		}

		// Delete a report (reportName is fully qualified)
		public static string DeleteReport(SessionInfo sessionInfo, string reportName)
		{
            string[] pathParts = reportName.Split(tokens);
			string parentIdStr = GetParentFolderIdString(sessionInfo.CompanyId, sessionInfo.UserId, pathParts, true);

			// NOTE: In your own implementation, use companyId and userId to enforce edit permissions on files and folders
            string stmt = String.Format("UPDATE {0} SET isDeleted = 'true' WHERE name=@name AND {1}", AssemblyExample.reportTableName, parentIdStr);

			List<SqlParameter> sqlParams = new List<SqlParameter>();
			sqlParams.Add(new SqlParameter("@name", pathParts[pathParts.Length - 1]));

			return ExecuteNQCmd(stmt, sqlParams);
		}

		// Renames a report (reportName and newReportName are fully qualified)
		public static string RenameReport(SessionInfo sessionInfo, string reportName, string newReportName)
		{

            string[] oldPathParts = reportName.Split(tokens);
			string[] newPathParts = newReportName.Split(tokens);
			string parentIdStr = GetParentFolderIdString(sessionInfo.CompanyId, sessionInfo.UserId, oldPathParts, true);

			// NOTE: In your own implementation, use companyId and userId to enforce edit permissions on files and folders
            string stmt = String.Format("UPDATE {0} SET name=@newName WHERE name=@oldName AND {1}", AssemblyExample.reportTableName, parentIdStr);

			List<SqlParameter> sqlParams = new List<SqlParameter>();
			sqlParams.Add(new SqlParameter("@newName", newPathParts[newPathParts.Length - 1]));
			sqlParams.Add(new SqlParameter("@oldName", oldPathParts[oldPathParts.Length - 1]));

			return ExecuteNQCmd(stmt, sqlParams);
		}

		// Gets a list of the filenames of all the templates currently stored
		public static List<string> GetTemplateList(SessionInfo sessionInfo)
		{
			// NOTE: In your own implementation, use companyId and userId to enforce read permissions on templates
            string stmt = String.Format("SELECT name FROM {0} WHERE type='{1} AND isDeleted='false'' ORDER BY name ASC", AssemblyExample.reportTableName, TypeIds.TemplateType);
			DataSet ds = ExecuteReadCmd(stmt, new List<SqlParameter>());

			List<string> retVal = new List<string>();

			foreach (DataRow r in ds.Tables[0].Rows)
				retVal.Add(r["name"].ToString());

			return retVal;
		}

		// Gets the binary content of a template
		public static byte[] GetTemplate(SessionInfo sessionInfo, string templateName)
		{
			// NOTE: In your own implementation, use companyId and userId to enforce read permissions on templates
            string stmt = String.Format("SELECT bin_content FROM {0} WHERE type='{1}' AND name=@templateName", AssemblyExample.reportTableName, TypeIds.TemplateType);

			List<SqlParameter> sqlParams = new List<SqlParameter>();
			sqlParams.Add(new SqlParameter("@templateName", templateName));

			DataSet ds = ExecuteReadCmd(stmt, sqlParams);
			byte[] retVal = (byte[])ds.Tables[0].Rows[0]["bin_content"];

			return retVal;
		}


		// Save a template
		public static string SaveTemplate(SessionInfo sessionInfo, string templateName, byte[] templateData)
		{
            // NOTE: In your own implementation, use companyId and userId to enforce write permissions on templates
            string chStmt = String.Format("SELECT name FROM {0} WHERE name=@templateName AND type='{1}'", AssemblyExample.reportTableName, TypeIds.TemplateType);

			List<SqlParameter> sqlParams = new List<SqlParameter>();
			sqlParams.Add(new SqlParameter("@templateName", templateName));

			Object chkObj = ExecuteScalarCmd(chStmt, sqlParams);

			if (chkObj != null)
				return "Error saving template file: File exists";

            string stmt = String.Format("INSERT INTO {0} VALUES (@companyId, @userId, @templateName, NULL, 'true', 'false', @templateData, '{1}', 'false', NULL, NULL)", AssemblyExample.reportTableName, TypeIds.TemplateType);

			sqlParams.Clear();
			sqlParams.Add(new SqlParameter("@templateName", templateName));
			sqlParams.Add(new SqlParameter("@templateData", templateData));
            sqlParams.Add(new SqlParameter("@companyId", sessionInfo.CompanyId));
            sqlParams.Add(new SqlParameter("@userId", sessionInfo.UserId));
			return ExecuteNQCmd(stmt, sqlParams);
		}

		// Gets a list of the filenames of all the themes currently stored
		public static List<string> GetThemeList(SessionInfo sessionInfo, string themeType)
		{
			// NOTE: In your own implementation, use companyId and userId to enforce read permissions on themes
            string stmt = String.Format("SELECT name FROM {0} WHERE type='{1}' AND themeType =@themeType ORDER BY name ASC", AssemblyExample.reportTableName, TypeIds.ThemeType);
            List<SqlParameter> sqlParams = new List<SqlParameter>();
            sqlParams.Add(new SqlParameter("@themeType", themeType));
            DataSet ds = ExecuteReadCmd(stmt, sqlParams);


			List<string> retVal = new List<string>();
			foreach (DataRow r in ds.Tables[0].Rows)
				retVal.Add(r["name"].ToString());

			return retVal;
		}

		// Determines if a theme exists; returns true or false
		public static bool ExistTheme(SessionInfo sessionInfo, string themeType, string themeName)
		{
			// NOTE: In your own implementation, use companyId and userId to enforce read permissions on themes
            string stmt = String.Format("SELECT name FROM {0} WHERE name=@name AND type='{1} AND isDeleted='false''", AssemblyExample.reportTableName, TypeIds.ThemeType);

			List<SqlParameter> sqlParams = new List<SqlParameter>();
			sqlParams.Add(new SqlParameter("@name", themeName));

			object obj = ExecuteScalarCmd(stmt, sqlParams);

			return (obj != null);
		}

		// Gets a theme
		public static string GetThemeXml(SessionInfo sessionInfo, string themeType, string themeName)
		{
			// NOTE: In your own implementation, use companyId and userId to enforce read permissions on themes
            string stmt = String.Format("SELECT content FROM {0} WHERE name=@name AND type='{1}' AND themeType = @themeType", AssemblyExample.reportTableName, TypeIds.ThemeType);

			List<SqlParameter> sqlParams = new List<SqlParameter>();
			sqlParams.Add(new SqlParameter("@name", themeName));
            sqlParams.Add(new SqlParameter("@themeType", themeType));
			object obj = ExecuteScalarCmd(stmt, sqlParams);
            
            return obj.ToString();
		}

		// Save a theme
		public static string SaveTheme(SessionInfo sessionInfo, string themeType, string themeName, string themeXml)
		{
			string stmt = "";

			// NOTE: In your own implementation, use companyId and userId to enforce write permissions on themes
            string chStmt = String.Format("SELECT name FROM {0} WHERE name=@name AND type='{1}'", AssemblyExample.reportTableName, TypeIds.ThemeType);

			List<SqlParameter> sqlParams = new List<SqlParameter>();
			sqlParams.Add(new SqlParameter("@name", themeName));

			Object chkObj = ExecuteScalarCmd(chStmt, sqlParams);

			sqlParams.Clear();

			if (chkObj == null)
			{
                stmt = String.Format("INSERT INTO {0} VALUES ( @companyId, @userId, @name, @themeXml, 'true', 'false', NULL, '{1}', 'false', {2}, NULL )", AssemblyExample.reportTableName, TypeIds.ThemeType, themeType);
				sqlParams.Add(new SqlParameter("@companyId", sessionInfo.CompanyId));
				sqlParams.Add(new SqlParameter("@userId", sessionInfo.UserId));
				sqlParams.Add(new SqlParameter("@name", themeName));
				sqlParams.Add(new SqlParameter("@themeXml", themeXml));
			}
			else
			{
                stmt = String.Format("UPDATE {0} SET content = @themeXml WHERE name=@name", AssemblyExample.reportTableName);
				sqlParams.Add(new SqlParameter("@themeXml", themeXml));
				sqlParams.Add(new SqlParameter("@name", themeName));
			}
			return ExecuteNQCmd(stmt, sqlParams);
		}

		// Used to simplify query logic when dealing with parent folders
		private static string GetParentFolderIdString(string companyId, string userId, string[] pathParts, bool isQuery)
		{
			if (pathParts.Length < 2) //root folder, no parent ID
			{
				if (isQuery)
					return "PARENT_ID is NULL";
				else
					return "NULL";
			}
			else
			{
				int parentId = GetParentFolderId(companyId, userId, pathParts);
				if (parentId < 0) //root folder, add with ParentId == null
				{
					if (isQuery)
						return "PARENT_ID is NULL";
					else
						return "NULL";
				}
				else
				{
					if (isQuery)
						return "PARENT_ID='" + parentId + "'";
					else
						return "'" + parentId + "'";
				}
			}
		}

		// Returns the ID of the parent folder of the report
		private static int GetParentFolderId(string companyId, string userId, string[] pathParts)
		{
			int retVal = -1;  //returned if pathParts is a root folder or path not found
            string cmd = String.Format("SELECT id FROM {0} WHERE name=@name AND leaf_flag='false'", AssemblyExample.reportTableName);

			List<SqlParameter> sqlParams = new List<SqlParameter>();
			sqlParams.Add(new SqlParameter("@name", pathParts[0]));

			object obj = ExecuteScalarCmd(cmd, sqlParams);

			if (obj != null)
			{
				string objString = obj.ToString();
				retVal = Int32.Parse(objString);

				for (int i = 1; i < pathParts.Length - 1; i++)
				{
                    cmd = String.Format("SELECT Id FROM {0} WHERE name=@name AND PARENT_ID = @retVal AND leaf_flag='false'", AssemblyExample.reportTableName);

					sqlParams.Clear();
					sqlParams.Add(new SqlParameter("@name", pathParts[i]));
					sqlParams.Add(new SqlParameter("@retVal", retVal));

					retVal = Int32.Parse(ExecuteScalarCmd(cmd, sqlParams).ToString());
				}
			}

			return retVal;
		}


		/// <summary>Executes a SQL query (meant for use with INSERT or UPDATE statements)</summary>
		/// <param name="stmt">The query to execute</param>
		/// <param name="paramList">A list of SqlParameters to substitute into the query string</param>
		/// <returns>null, or an exception string</returns>
		private static string ExecuteNQCmd(string stmt, List<SqlParameter> paramList)
		{
			try
			{
				using (SqlConnection sqlConn = InitConnection())
				{
					SqlCommand command = new SqlCommand(stmt, sqlConn);

					foreach (SqlParameter p in paramList)
						command.Parameters.Add(p);
					command.ExecuteNonQuery();
				}
			}
			catch (Exception e)
			{
				return e.ToString();
			}

			return null;
		}

		/// <summary>Executes a SQL query (meant for use with SELECT statements returning a single value)</summary>
		/// <param name="stmt">The query to execute</param>
		/// <param name="paramList">A list of SqlParameters to substitute into the query string</param>
		/// <returns>The data object resulting from the query, or an exception string in case of an exception</returns>
		private static Object ExecuteScalarCmd(string stmt, List<SqlParameter> paramList)
		{
			object retVal = null;

			try
			{
				using (SqlConnection sqlConn = InitConnection())
				{
					SqlCommand command = new SqlCommand(stmt, sqlConn);

					foreach (SqlParameter p in paramList)
						command.Parameters.Add(p);

					retVal = command.ExecuteScalar();
				}
			}
			catch (Exception e)
			{
				return e.ToString();
			}

			return retVal;
		}

		/// <summary>Executes a SQL query (meant for use with SELECT statements)</summary>
		/// <param name="stmt">The query to execute</param>
		/// <param name="paramList">A list of SqlParameters to substitute into the query string</param>
		/// <returns>The data resulting from the query, or null in case of an exception</returns>
		private static DataSet ExecuteReadCmd(string stmt, List<SqlParameter> paramList)
		{
			DataSet retVal = new DataSet();

			try
			{
				using (SqlConnection sqlConn = InitConnection())
				{
					SqlDataAdapter adapter = new SqlDataAdapter(stmt, sqlConn);

					foreach (SqlParameter p in paramList)
						adapter.SelectCommand.Parameters.Add(p);

					adapter.Fill(retVal);
				}
			}
			catch
			{
				return null;
			}

			return retVal;
		}
    }
}
