﻿/*
 * Sample Folder Management Web Service
 * Exago, Inc
 * support@exagoinc.com
 * 
 * This file is provided as an example web service for report folder and report template management within eWebReports.
 * Although this web service should provide a good starting point, it is recommended that you add/modify code to fully integrate
 * with your system to account for application standards including error handling and database design.  Specifically, the parameters
 * userId and companyId passed to all methods should be used to enforce management permissions on a user and/or company basis.
 * 
 * Please make sure to review/alter this code to make sure it meets your needs before integrating into a production system!
 *
 * ---------------------------------------------------------------------------------------------------------------
 *   PLEASE NOTE
 *   This software is provided to our clients as a convenience "as-is," with no implied warranty.
 * ---------------------------------------------------------------------------------------------------------------
 * 
 * Quick setup:
 * 
 * 1. Create the database table to store the report, folder, and template data (schema info below)
 * 2. Fill in the example database connection info in this file with your own (see member variables eg. "this.server") and build the solution
 * 3. Tell eWebReports to use the web service for folder management
 *    a) Open the admin console
 *    b) Navigate to General -> Main Settings
 *    c) Change the "Report Path" field to point to the .asmx file associated with this code behind, using the "url=" scheme.
 *       eg. url=http://localhost/ExagoWebServiceExample/ExagoWebServiceExample.asmx 
 * 4. Once the Report Path field validates without error, eWebReports will use the database for storage of reports, folders, and templates
 * 
 * 
 * Note: This example was developed and tested against MSSQL Server, however, the queries used in the example 
 * methods provided should work across major database platforms with equivalent schemas.
 * 
 * Database Schema:
 * 
 * ID            int identity...Unique ID
 * PARENT_ID     int............ID of the parent folder of the report (null for folders and templates)
 * companyId     varchar........Company ID of the owner of the report (null for templates)
 * userId        varchar........User ID of the owner of the report (null for templates)
 * name          varchar........Name of the report, template, or folder
 * content       varchar........XML content of the report (null for folders and templates)
 * leaf_flag     bit............True for reports and templates, false for folders
 * readonly_flag bit............True if the report or folder is readonly
 * bin_content   varbinary......Contains the template file data (null if not a template)
 * type          int............0 if report, 1 if template, 2 if theme
 * 
 * CREATE TABLE Tablename(ID int identity, PARENT_ID int, companyId varchar, userId varchar, name varchar, 
 *                     content varchar, leaf_flag bit, readonly_flag bit, bin_content varbinary(max), type int);
 * 
 */

using System;
using System.Web;
using System.Web.Services;
using System.Web.Services.Protocols;
using System.Data;
using System.Xml;
using System.IO;
using System.Data.SqlClient;
using System.Collections.Generic;
using WebReports.Api.Common;

namespace ExagoFolderMgmtExamples
{
	[WebService(Namespace = "http://localhost/")]
	[WebServiceBinding(ConformsTo = WsiProfiles.BasicProfile1_1)]
	public class WebServiceExample : System.Web.Services.WebService
	{
		// Return XML representing the hierarchy of reports and folders
		[WebMethod]
		public string GetReportListXml(SessionInfo sessionInfo)
		{
			return AssemblyExample.GetReportListXml(sessionInfo);
		}

		// Add a report folder (folderName is fully qualified). Folder should not be named to one that already exists
		[WebMethod]
		public string AddFolder(SessionInfo sessionInfo, string companyId, string userId, string folderName)
		{
			return AssemblyExample.AddFolder(sessionInfo, folderName);
		}

		// Delete a report folder (folderName is fully qualified)
		[WebMethod]
		public string DeleteFolder(SessionInfo sessionInfo, string companyId, string userId, string folderName)
		{
			return AssemblyExample.DeleteFolder(sessionInfo,  folderName);
		}

		// Rename a report folder (folderName is fully qualified). Folder should not be renamed to one that already exists
		[WebMethod]
		public string RenameFolder(SessionInfo sessionInfo, string companyId, string userId, string oldName, string newName)
		{
			return AssemblyExample.RenameFolder(sessionInfo, oldName, newName);
		}

		// Determines if a folder exists; returns true or false
		[WebMethod]
		public bool ExistFolder(SessionInfo sessionInfo, string companyId, string userId, string folderName)
		{
			return AssemblyExample.ExistFolder(sessionInfo, folderName);
		}

		// Gets a report (reportName is fully qualified)
		[WebMethod]
		public string GetReportXml(SessionInfo sessionInfo, string reportName)
		{
			return AssemblyExample.GetReportXml(sessionInfo, reportName);
		}

		// Save a report (reportName is fully qualified)
		[WebMethod]
		public string SaveReport(SessionInfo sessionInfo, string companyId, string userId, string reportName, string reportXml)
		{
			return AssemblyExample.SaveReport(sessionInfo, reportName, reportXml);
		}

		// Duplicates a report (reportName is fully qualified). If this method is not provided, SaveReport will be called
		public string DuplicateReport(SessionInfo sessionInfo, string companyId, string userId, string reportName, string reportXml)
		{
			return AssemblyExample.DuplicateReport(sessionInfo, reportName, reportXml);
		}

		// Delete a report (reportName is fully qualified)
		[WebMethod]
		public string DeleteReport(SessionInfo sessionInfo, string companyId, string userId, string reportName)
		{
			return AssemblyExample.DeleteReport(sessionInfo, reportName);
		}

		// Renames a report (reportName and newReportName are fully qualified)
		[WebMethod]
		public string RenameReport(SessionInfo sessionInfo, string companyId, string userId, string reportName, string newReportName)
		{
			return AssemblyExample.RenameReport(sessionInfo, reportName, newReportName);
		}

		// Gets a list of the filenames of all the templates currently stored
		[WebMethod]
		public List<string> GetTemplateList(SessionInfo sessionInfo, string companyId, string userId)
		{
			return AssemblyExample.GetTemplateList(sessionInfo);
		}

		// Gets the binary content of a template
		[WebMethod]
		public byte[] GetTemplate(SessionInfo sessionInfo, string companyId, string userId, string templateName)
		{
			return AssemblyExample.GetTemplate(sessionInfo, templateName);
		}


		// Save a template
		[WebMethod]
		public string SaveTemplate(SessionInfo sessionInfo, string companyId, string userId, string templateName, byte[] templateData)
		{
			return AssemblyExample.SaveTemplate(sessionInfo, templateName, templateData);
		}

		// Gets a list of the filenames of all the themes currently stored
		[WebMethod]
		public List<string> GetThemeList(SessionInfo sessionInfo, string companyId, string userId, string themeType)
		{
			return AssemblyExample.GetThemeList(sessionInfo, themeType);
		}

		// Determines if a theme exists; returns true or false
		[WebMethod]
		public bool ExistTheme(SessionInfo sessionInfo, string companyId, string userId, string themeType, string themeName)
		{
			return AssemblyExample.ExistTheme(sessionInfo, themeType, themeName);
		}

		// Gets a theme
		[WebMethod]
		public string GetThemeXml(SessionInfo sessionInfo, string companyId, string userId, string themeType, string themeName)
		{
			return AssemblyExample.GetThemeXml(sessionInfo, themeType, themeName);
		}

		// Save a theme
		[WebMethod]
		public string SaveTheme(SessionInfo sessionInfo, string companyId, string userId, string themeType, string themeName, string themeXml)
		{
			return AssemblyExample.SaveTheme(sessionInfo, themeType, themeName, themeXml);
        }
    }
}
